@@ -59,25 +59,30 @@ unsigned int    PrimeInstance::m_recovery_count = 0;
 PrimeInstance::PrimeInstance (PrimeFactory   *factory,
                               const String   &encoding,
                               int             id)
-    : IMEngineInstanceBase (factory, encoding, id),
-      m_session (NULL),
-      m_factory (factory),
-      m_prev_key (0,0),
-      m_language (SCIM_PRIME_LANGUAGE_OFF),
-      m_disable (false),
-      m_converting (false),
-      m_modifying (false),
-      m_registering (false),
-      m_cancel_prediction (false),
-      m_preedition_visible (false),
+    : IMEngineInstanceBase   (factory, encoding, id),
+      m_session              (NULL),
+      m_factory              (factory),
+      m_prev_key             (0,0),
+      m_language             (SCIM_PRIME_LANGUAGE_OFF),
+      m_disable              (false),
+      m_converting           (false),
+      m_modifying            (false),
+      m_registering          (false),
+      m_cancel_prediction    (false),
+      m_preedition_visible   (false),
       m_lookup_table_visible (false),
-      m_registering_cursor (0)
+      m_registering_cursor   (0)
 {
     SCIM_DEBUG_IMENGINE(1) << "Create PRIME Instance : ";
 
     if (!m_prime.is_connected ()) {
-        m_prime.open_connection (m_factory->m_command.c_str(),
-                                 m_factory->m_typing_method.c_str());
+        bool rv;
+        rv = m_prime.open_connection (m_factory->m_command.c_str(),
+                                      m_factory->m_typing_method.c_str());
+        if (!rv) {
+            m_disable = true;
+            set_error_message ();
+        }
     }
 }
 
@@ -115,6 +120,9 @@ PrimeInstance::process_key_event (const KeyEvent& key)
     // try recovery on losting connection.
     if (!get_session ()) {
         if (m_recovery_count <= MAX_TRY_RECOVERY) {
+            //
+            // automatical recovery
+            //
             action_recovery ();
             if (!get_session ()) {
                 m_recovery_count++;
@@ -123,6 +131,9 @@ PrimeInstance::process_key_event (const KeyEvent& key)
                 return false;
             }
         } else {
+            //
+            // intentional recovery
+            //
             if (m_factory->m_recovery_action &&
                 m_factory->m_recovery_action->perform (this, key))
             {
@@ -132,7 +143,6 @@ PrimeInstance::process_key_event (const KeyEvent& key)
             }
         }
     }
-
     m_recovery_count = 0;
 
     bool prediction_canceled = m_cancel_prediction;
@@ -147,7 +157,7 @@ PrimeInstance::process_key_event (const KeyEvent& key)
     if (prediction_canceled)
         m_cancel_prediction = false;
 
-    return process_remaining_key_event (key);
+    return process_input_key_event (key);
 }
 
 bool
@@ -163,7 +173,7 @@ PrimeInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 }
 
 bool
-PrimeInstance::process_remaining_key_event (const KeyEvent &key)
+PrimeInstance::process_input_key_event (const KeyEvent &key)
 {
     if (m_language == SCIM_PRIME_LANGUAGE_OFF)
         return false;
@@ -311,7 +321,7 @@ PrimeInstance::reset ()
 
     m_candidates.clear();
     m_converting = false;
-    m_modifying = false;
+    m_modifying  = false;
 
     m_preedition_visible   = false;
     m_lookup_table_visible = false;
@@ -365,11 +375,15 @@ PrimeInstance::focus_in (void)
 
     install_properties ();
 
-    if (m_preedition_visible)
-        set_preedition ();
-    if (m_lookup_table_visible) {
-        update_lookup_table (m_lookup_table);
-        show_lookup_table ();
+    if (m_disable) {
+        set_error_message ();
+    } else {
+        if (m_preedition_visible)
+            set_preedition ();
+        if (m_lookup_table_visible) {
+            update_lookup_table (m_lookup_table);
+            show_lookup_table ();
+        }
     }
 }
 
@@ -415,13 +429,7 @@ PrimeInstance::get_session (void)
         delete m_session;
         m_session = NULL;
         m_disable = true;
-
-        const char *message =
-            _("PRIME process seems terminated abnormally."
-              "Press Control+Alt+r to try recovery.");
-        show_aux_string ();
-        update_aux_string (utf8_mbstowcs (message));
-
+        set_error_message ();
         return NULL;
 
     } else if (m_prime.major_version () < 1) {
@@ -500,76 +508,7 @@ PrimeInstance::set_preedition (void)
         update_preedit_caret (0);
 
     } else if (is_preediting ()) {
-        WideString left, cursor, right;
-        get_session()->edit_get_preedition (left, cursor, right);
-
-        m_candidates.clear ();
-        if (m_factory->m_inline_prediction && !m_cancel_prediction)
-            get_session()->conv_predict (m_candidates);
-
-        if (left.length () + cursor.length () + right.length () > 0) {
-            show_preedit_string ();
-            m_preedition_visible = true;
-        } else {
-            hide_preedit_string ();
-            m_preedition_visible = false;
-        }
-
-        if (!m_candidates.empty ()) {
-            // on inline prediction mode
-
-            if (!left.empty () && cursor.empty () && right.empty ())
-                cursor = utf8_mbstowcs (" ");
-
-            AttributeList attr_list;
-            if (!cursor.empty ()) {
-                Attribute attr (0, m_candidates[0].m_conversion.length (),
-                                SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
-                attr_list.push_back (attr);
-            }
-
-            update_preedit_string (m_candidates[0].m_conversion, attr_list);
-            update_preedit_caret (0);
-
-            attr_list.clear ();
-            if (!cursor.empty ()) {
-                Attribute attr (left.length (), cursor.length(),
-                                SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
-                attr_list.push_back (attr);
-            }
-
-            show_aux_string ();
-            update_aux_string (left + cursor + right, attr_list);
-
-        } else {
-            // on normal prediction mode
-
-            bool place_cursor_head
-                =  m_factory->m_predict_on_preedition
-                && m_factory->m_predict_win_pos == "head";
-
-            if (place_cursor_head && !left.empty () &&
-                cursor.empty () && right.empty ())
-            {
-                cursor = utf8_mbstowcs (" ");
-            }
-
-            AttributeList attr_list;
-            if (!cursor.empty ()) {
-                Attribute attr (left.length (), cursor.length(),
-                                SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
-                attr_list.push_back (attr);
-            }
-
-            update_preedit_string (left + cursor + right, attr_list);
-            if (place_cursor_head)
-                update_preedit_caret (0);
-            else
-                update_preedit_caret (left.length ());
-
-            hide_aux_string ();
-            update_aux_string (utf8_mbstowcs (""));
-        }
+        set_preedition_on_preediting ();
 
     } else {
         reset ();
@@ -586,6 +525,81 @@ PrimeInstance::set_preedition (void)
     attr_list.push_back (attr);                 \
 }
 
+void
+PrimeInstance::set_preedition_on_preediting (void)
+{
+    WideString left, cursor, right;
+    get_session()->edit_get_preedition (left, cursor, right);
+
+    m_candidates.clear ();
+    if (m_factory->m_inline_prediction && !m_cancel_prediction)
+        get_session()->conv_predict (m_candidates);
+
+    if (left.length () + cursor.length () + right.length () > 0) {
+        show_preedit_string ();
+        m_preedition_visible = true;
+    } else {
+        hide_preedit_string ();
+        m_preedition_visible = false;
+    }
+
+    if (!m_candidates.empty ()) {
+        // on inline prediction mode
+
+        if (!left.empty () && cursor.empty () && right.empty ())
+            cursor = utf8_mbstowcs (" ");
+
+        AttributeList attr_list;
+        if (!cursor.empty ()) {
+            Attribute attr (0, m_candidates[0].m_conversion.length (),
+                            SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
+            attr_list.push_back (attr);
+        }
+
+        update_preedit_string (m_candidates[0].m_conversion, attr_list);
+        update_preedit_caret (0);
+
+        attr_list.clear ();
+        if (!cursor.empty ()) {
+            Attribute attr (left.length (), cursor.length(),
+                            SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
+            attr_list.push_back (attr);
+        }
+
+        show_aux_string ();
+        update_aux_string (left + cursor + right, attr_list);
+
+    } else {
+        // on normal prediction mode
+
+        bool place_cursor_head
+            =  m_factory->m_predict_on_preedition
+            && m_factory->m_predict_win_pos == "head";
+
+        if (place_cursor_head && !left.empty () &&
+            cursor.empty () && right.empty ())
+        {
+            cursor = utf8_mbstowcs (" ");
+        }
+
+        AttributeList attr_list;
+        if (!cursor.empty ()) {
+            Attribute attr (left.length (), cursor.length(),
+                            SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE);
+            attr_list.push_back (attr);
+        }
+
+        update_preedit_string (left + cursor + right, attr_list);
+        if (place_cursor_head)
+            update_preedit_caret (0);
+        else
+            update_preedit_caret (left.length ());
+
+        hide_aux_string ();
+        update_aux_string (utf8_mbstowcs (""));
+    }
+}
+
 void
 PrimeInstance::set_preedition_on_register (void)
 {
@@ -764,7 +778,7 @@ PrimeInstance::set_prediction (void)
         for (unsigned int i = 0; i < m_candidates.size (); i++)
             m_lookup_table.append_candidate (m_candidates[i].m_conversion);
 
-        unsigned int min_candidates = m_factory->m_inline_prediction ? 1: 0;
+        unsigned int min_candidates = m_factory->m_inline_prediction ? 1 : 0;
 
         if (m_candidates.size () > min_candidates &&
             m_candidates[0].m_conversion.length () > 0)
@@ -1394,7 +1408,8 @@ PrimeInstance::action_conv_next_candidate (void)
         m_lookup_table.cursor_down ();
     }
 
-    select_candidate_no_direct (m_lookup_table.get_cursor_pos_in_current_page ());
+    unsigned int pos = m_lookup_table.get_cursor_pos_in_current_page ();
+    select_candidate_no_direct (pos);
 
     return true;
 }
@@ -1439,7 +1454,8 @@ PrimeInstance::action_conv_prev_candidate (void)
         m_lookup_table.cursor_up ();
     }
 
-    select_candidate_no_direct (m_lookup_table.get_cursor_pos_in_current_page ());
+    unsigned int pos = m_lookup_table.get_cursor_pos_in_current_page ();
+    select_candidate_no_direct (pos);
 
     return true;
 }
@@ -1451,7 +1467,9 @@ PrimeInstance::action_conv_next_page (void)
         return false;
 
     m_lookup_table.page_down ();
-    select_candidate_no_direct (m_lookup_table.get_cursor_pos_in_current_page ());
+
+    unsigned int pos = m_lookup_table.get_cursor_pos_in_current_page ();
+    select_candidate_no_direct (pos);
 
     return true;
 }
@@ -1463,7 +1481,9 @@ PrimeInstance::action_conv_prev_page (void)
         return false;
 
     m_lookup_table.page_up ();
-    select_candidate_no_direct (m_lookup_table.get_cursor_pos_in_current_page ());
+
+    unsigned int pos = m_lookup_table.get_cursor_pos_in_current_page ();
+    select_candidate_no_direct (pos);
 
     return true;
 }
@@ -2001,12 +2021,17 @@ PrimeInstance::action_recovery (void)
     if (!m_disable)
         return false;
 
-    m_prime.open_connection (m_factory->m_command.c_str(),
-                             m_factory->m_typing_method.c_str());
-    m_disable = false;
-    update_aux_string (utf8_mbstowcs(""));
-    hide_aux_string ();
-
+    bool rv;
+    rv = m_prime.open_connection (m_factory->m_command.c_str(),
+                                  m_factory->m_typing_method.c_str());
+    if (rv) {
+        m_disable = false;
+        update_aux_string (utf8_mbstowcs(""));
+        hide_aux_string ();
+    } else {
+        m_disable = true;
+        set_error_message ();
+    }
     return true;
 }
 
@@ -2023,7 +2048,9 @@ PrimeInstance::match_key_event (const KeyEventList &keys, const KeyEvent &key) c
 }
 
 void
-PrimeInstance::get_candidate_label (WideString &label, AttributeList &attrs, PrimeCandidate &cand)
+PrimeInstance::get_candidate_label (WideString &label,
+                                    AttributeList &attrs,
+                                    PrimeCandidate &cand)
 {
     label = cand.m_conversion;
     attrs.clear ();
@@ -2063,6 +2090,26 @@ PrimeInstance::get_candidate_label (WideString &label, AttributeList &attrs, Pri
                                     m_factory->m_candidate_comment_color));
     }
 }
+
+void
+PrimeInstance::set_error_message (void)
+{
+    WideString msg;
+    m_prime.get_error_message (msg);
+
+    update_aux_string (msg);
+    show_aux_string ();
+
+    install_properties ();
+    PropertyList::iterator it = std::find (m_properties.begin (),
+                                           m_properties.end (),
+                                           SCIM_PROP_LANGUAGE);
+    if (it != m_properties.end ()) {
+        it->set_label (_("Disabled"));
+        it->set_tip (utf8_wcstombs (msg));
+        update_property (*it);
+    }
+}
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
